@@ -24,6 +24,7 @@
 #include <string.h>
 #include "flash.h"
 #include "programmer.h"
+#include "hwaccess.h"
 
 #define BIOS_ROM_ADDR		0x90
 #define BIOS_ROM_DATA		0x94
@@ -59,7 +60,6 @@ static int atahpt_shutdown(void *data)
 {
 	/* Flash access is disabled automatically by PCI restore. */
 	pci_cleanup(pacc);
-	release_io_perms();
 	return 0;
 }
 
@@ -67,7 +67,8 @@ int atahpt_init(void)
 {
 	uint32_t reg32;
 
-	get_io_perms();
+	if (rget_io_perms())
+		return 1;
 
 	io_base_addr = pcidev_init(PCI_BASE_ADDRESS_4, ata_hpt);
 
