@@ -39,7 +39,8 @@ static int buspirate_serialport_setup(char *dev)
 {
 	/* 115200bps, 8 databits, no parity, 1 stopbit */
 	sp_fd = sp_openserport(dev, 115200);
-	/* FIXME: Error checking */
+ 	if (sp_fd < 0)
+		return 1;
 	return 0;
 }
 #else
