@@ -29,6 +29,7 @@
 #include <stdlib.h>
 #include "flash.h"
 #include "programmer.h"
+#include "hwaccess.h"
 #include "spi.h"
 #include "ich_descriptors.h"
 
@@ -405,18 +406,16 @@ static int reprogram_opcode_on_the_fly(uint8_t opcode, unsigned int writecnt, un
 			spi_type = SPI_OPCODE_TYPE_READ_NO_ADDRESS;
 		else if (writecnt == 4) // and readcnt is > 0
 			spi_type = SPI_OPCODE_TYPE_READ_WITH_ADDRESS;
-		// else we have an invalid case, will be handled below
+		else // we have an invalid case
+			return SPI_INVALID_LENGTH;
 	}
-	if (spi_type <= 3) {
-		int oppos=2;	// use original JEDEC_BE_D8 offset
-		curopcodes->opcode[oppos].opcode = opcode;
-		curopcodes->opcode[oppos].spi_type = spi_type;
-		program_opcodes(curopcodes, 0);
-		oppos = find_opcode(curopcodes, opcode);
-		msg_pdbg ("on-the-fly OPCODE (0x%02X) re-programmed, op-pos=%d\n", opcode, oppos);
-		return oppos;
-	}
-	return -1;
+	int oppos = 2;	// use original JEDEC_BE_D8 offset
+	curopcodes->opcode[oppos].opcode = opcode;
+	curopcodes->opcode[oppos].spi_type = spi_type;
+	program_opcodes(curopcodes, 0);
+	oppos = find_opcode(curopcodes, opcode);
+	msg_pdbg2("on-the-fly OPCODE (0x%02X) re-programmed, op-pos=%d\n", opcode, oppos);
+	return oppos;
 }
 
 static int find_opcode(OPCODES *op, uint8_t opcode)
@@ -536,7 +535,7 @@ static int program_opcodes(OPCODES *op, int enable_undo)
 		opmenu[1] |= ((uint32_t) op->opcode[a].opcode) << ((a - 4) * 8);
 	}
 
-	msg_pdbg("\n%s: preop=%04x optype=%04x opmenu=%08x%08x\n", __func__, preop, optype, opmenu[0], opmenu[1]);
+	msg_pdbg2("\n%s: preop=%04x optype=%04x opmenu=%08x%08x\n", __func__, preop, optype, opmenu[0], opmenu[1]);
 	switch (ich_generation) {
 	case CHIPSET_ICH7:
 		/* Register undo only for enable_undo=1, i.e. first call. */
@@ -1001,7 +1000,10 @@ static int ich_spi_send_command(struct flashctx *flash, unsigned int writecnt,
 	if (opcode_index == -1) {
 		if (!ichspi_lock)
 			opcode_index = reprogram_opcode_on_the_fly(cmd, writecnt, readcnt);
-		if (opcode_index == -1) {
+		if (opcode_index == SPI_INVALID_LENGTH) {
+			msg_pdbg("OPCODE 0x%02x has unsupported length, will not execute.\n", cmd);
+			return SPI_INVALID_LENGTH;
+		} else if (opcode_index == -1) {
 			msg_pdbg("Invalid OPCODE 0x%02x, will not execute.\n",
 				 cmd);
 			return SPI_INVALID_OPCODE;
@@ -1520,6 +1522,7 @@ static const struct spi_programmer spi_programmer_ich7 = {
 	.multicommand = ich_spi_send_multicommand,
 	.read = default_spi_read,
 	.write_256 = default_spi_write_256,
+	.write_aai = default_spi_write_aai,
 };
 
 static const struct spi_programmer spi_programmer_ich9 = {
@@ -1530,6 +1533,7 @@ static const struct spi_programmer spi_programmer_ich9 = {
 	.multicommand = ich_spi_send_multicommand,
 	.read = default_spi_read,
 	.write_256 = default_spi_write_256,
+	.write_aai = default_spi_write_aai,
 };
 
 static const struct opaque_programmer opaque_programmer_ich_hwseq = {
@@ -1837,6 +1841,7 @@ static const struct spi_programmer spi_programmer_via = {
 	.multicommand = ich_spi_send_multicommand,
 	.read = default_spi_read,
 	.write_256 = default_spi_write_256,
+	.write_aai = default_spi_write_aai,
 };
 
 int via_init_spi(struct pci_dev *dev)
