@@ -28,6 +28,7 @@
 #include <stdlib.h>
 #include "flash.h"
 #include "programmer.h"
+#include "hwaccess.h"
 
 #define PCI_VENDOR_ID_INTEL 0x8086
 
@@ -158,7 +159,6 @@ static int nicintel_spi_shutdown(void *data)
 
 	physunmap(nicintel_spibar, 4096);
 	pci_cleanup(pacc);
-	release_io_perms();
 
 	return 0;
 }
@@ -167,7 +167,8 @@ int nicintel_spi_init(void)
 {
 	uint32_t tmp;
 
-	get_io_perms();
+	if (rget_io_perms())
+		return 1;
 
 	io_base_addr = pcidev_init(PCI_BASE_ADDRESS_0, nics_intel_spi);
 
