@@ -327,21 +327,24 @@ void spi_prettyprint_status_register_welwip(uint8_t status)
 }
 
 /* Prettyprint the status register. Common definitions. */
-void spi_prettyprint_status_register_bp3210(uint8_t status, int bp)
+void spi_prettyprint_status_register_bp(uint8_t status, int bp)
 {
 	switch (bp) {
 	/* Fall through. */
+	case 4:
+		msg_cdbg("Chip status register: Block Protect 4 (BP4) "
+			     "is %sset\n", (status & (1 << 5)) ? "" : "not ");
 	case 3:
-		msg_cdbg("Chip status register: Bit 5 / Block Protect 3 (BP3) "
+		msg_cdbg("Chip status register: Block Protect 3 (BP3) "
 			     "is %sset\n", (status & (1 << 5)) ? "" : "not ");
 	case 2:
-		msg_cdbg("Chip status register: Bit 4 / Block Protect 2 (BP2) "
+		msg_cdbg("Chip status register: Block Protect 2 (BP2) "
 			     "is %sset\n", (status & (1 << 4)) ? "" : "not ");
 	case 1:
-		msg_cdbg("Chip status register: Bit 3 / Block Protect 1 (BP1) "
+		msg_cdbg("Chip status register: Block Protect 1 (BP1) "
 			     "is %sset\n", (status & (1 << 3)) ? "" : "not ");
 	case 0:
-		msg_cdbg("Chip status register: Bit 2 / Block Protect 0 (BP0) "
+		msg_cdbg("Chip status register: Block Protect 0 (BP0) "
 			     "is %sset\n", (status & (1 << 2)) ? "" : "not ");
 	}
 }
@@ -355,7 +358,7 @@ void spi_prettyprint_status_register_bit(uint8_t status, int bit)
 
 static void spi_prettyprint_status_register_common(uint8_t status)
 {
-	spi_prettyprint_status_register_bp3210(status, 3);
+	spi_prettyprint_status_register_bp(status, 3);
 	spi_prettyprint_status_register_welwip(status);
 }
 
@@ -930,10 +933,10 @@ int spi_disable_blockprotect(struct flashctx *flash)
 	if ((status & 0x3c) == 0)
 		return 0;
 
-	msg_cdbg("Some block protection in effect, disabling\n");
+	msg_cdbg("Some block protection in effect, disabling... ");
 	result = spi_write_status_register(flash, status & ~0x3c);
 	if (result) {
-		msg_cerr("spi_write_status_register failed\n");
+		msg_cerr("spi_write_status_register failed.\n");
 		return result;
 	}
 	status = spi_read_status_register(flash);
@@ -941,6 +944,7 @@ int spi_disable_blockprotect(struct flashctx *flash)
 		msg_cerr("Block protection could not be disabled!\n");
 		return 1;
 	}
+	msg_cdbg("done.\n");
 	return 0;
 }
 
